'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/* 

high performance StyleSheet for css-in-js systems 

- uses `insertRule` in production for *much* faster performance
- 'polyfills' on server side in memory 

// usage
import StyleSheet from 'glamor/lib/sheet'
let styleSheet = new StyleSheet()

OR 

// pass a custom name/id for tag, and toggle insertRule manually 
let styleSheet = new StyleSheet({ name, speedy = true/false })

styleSheet.inject() 
// 'injects' the stylesheet into the page (or into memory if on server)

styleSheet.insert('#box { border: 1px solid red; }' [, index]) 
// inserts a css rule into the stylesheet 

styleSheet.sheet.cssRules 
// array-like collection of css rules 
// use for server side rendering, etc 

styesheet.remove(index)
// remove rule at position `index`

styleSheet.flush() 
// empties the stylesheet of all its contents


*/

/**** stylesheet ****/

var isBrowser = typeof document !== 'undefined';
var isDev = function (x) {
  return x === 'development' || !x;
}(process.env.NODE_ENV);
var isTest = process.env.NODE_ENV === 'test';

var tagCounter = 0;

var StyleSheet = exports.StyleSheet = function () {
  function StyleSheet(_ref) {
    var _ref$name = _ref.name;
    var name = _ref$name === undefined ? '_css_' + tagCounter++ : _ref$name;
    var _ref$speedy = _ref.speedy;
    var speedy = _ref$speedy === undefined ? !isDev && !isTest : _ref$speedy;

    _classCallCheck(this, StyleSheet);

    this.name = name;
    this.speedy = speedy; // the big drawback here is that the css won't be editable in devtools
    this.sheet = undefined;
    this.tag = undefined;
  }

  _createClass(StyleSheet, [{
    key: 'inject',
    value: function inject() {
      var _this = this;

      if (this.injected) {
        throw new Error('already injected stylesheet!'); //eslint-disable-line no-console
      }
      if (isBrowser) {
        // this section is just weird alchemy I found online off many sources 
        // it checks to see if the tag exists; creates an empty one if not 
        this.tag = document.getElementById(this.name);
        if (!this.tag) {
          var tag = document.createElement('style');

          tag.type = 'text/css';
          tag.id = this.name;
          tag.setAttribute('id', this.name);
          tag.appendChild(document.createTextNode(''));
          (document.head || document.getElementsByTagName('head')[0]).appendChild(tag);
          this.tag = tag;
        }
        // this weirdness brought to you by firefox 
        this.sheet = [].concat(_toConsumableArray(document.styleSheets)).filter(function (x) {
          return x.ownerNode === _this.tag;
        })[0];
      } else {
        // server side 'polyfill'. just enough behavior to be useful.
        this.sheet = {
          cssRules: [],
          deleteRule: function deleteRule(index) {
            _this.sheet.cssRules = [].concat(_toConsumableArray(_this.sheet.cssRules.slice(0, index)), _toConsumableArray(_this.sheet.cssRules.slice(index + 1)));
          },
          insertRule: function insertRule(rule) {
            var index = arguments.length <= 1 || arguments[1] === undefined ? _this.sheet.cssRules.length : arguments[1];

            // enough 'spec compliance' to be able to extract the rules later  
            // in other words, just the cssText field 
            _this.sheet.cssRules = [].concat(_toConsumableArray(_this.sheet.cssRules.slice(0, index)), [{ cssText: rule }], _toConsumableArray(_this.sheet.cssRules.slice(index)));
          }
        };
      }
      this.injected = true;
    }
  }, {
    key: '_insert',
    value: function _insert(rule, index) {
      // this weirdness for perf, and chrome's weird bug 
      // https://stackoverflow.com/questions/20007992/chrome-suddenly-stopped-accepting-insertrule

      try {
        this.sheet.insertRule(rule, index);
      } catch (e) {
        if (isDev) {
          // might need beter dx for this 
          console.warn('whoops, illegal rule inserted', rule); //eslint-disable-line no-console
        }
      }
    }
  }, {
    key: 'insert',
    value: function insert(rule) {
      var _this2 = this;

      var index = arguments.length <= 1 || arguments[1] === undefined ? this.sheet.cssRules.length : arguments[1];

      // more browser weirdness. I don't even know
      if (this.tag && this.tag.styleSheet) {
        this.tag.styleSheet.cssText += rule;
      } else {
        if (isBrowser) {
          if (this.speedy && this.sheet.insertRule) {
            this._insert(rule, index);
          } else {
            var refNode = this.tag.childNodes[index]; // not index - 1 as expected, because we have a blank leading node 
            refNode.parentNode.insertBefore(document.createTextNode(rule), refNode.nextSibling);

            // todo - more efficent here please 
            if (!this.speedy) {
              // sighhh
              this.sheet = [].concat(_toConsumableArray(document.styleSheets)).filter(function (x) {
                return x.ownerNode === _this2.tag;
              })[0];
            }
          }
        } else {
          // server side is pretty simple 
          this.sheet.insertRule(rule, index);
        }
      }
    }
  }, {
    key: 'flush',
    value: function flush() {
      // todo backward compat (styleTag.styleSheet.cssText?)
      if (isBrowser) {
        this.tag && this.tag.parentNode.removeChild(this.tag);
        this.tag = null;
        // todo - look for remnants in document.styleSheets
      } else {
        // simpler on server 
        this.sheet.cssRules = [];
      }
      this.injected = false;
    }
  }, {
    key: 'remove',
    value: function remove(i) {
      var _this3 = this;

      // todo - tests
      if (this.speedy || !isBrowser) {
        this.sheet.deleteRule(i);
      } else {
        this.tag.removeChild(this.stag.childNodes[i + 1]); // the +1 to account for the blank node we added
        // reassign stylesheet, because firefox is weird 
        this.sheet = [].concat(_toConsumableArray(document.styleSheets)).filter(function (x) {
          return x.ownerNode === _this3.tag;
        })[0];
      }
    }
  }]);

  return StyleSheet;
}();